package com.takensoft.cms.author.web;

import com.takensoft.cms.author.dto.AuthorDTO;
import com.takensoft.cms.author.service.AuthorService;
import com.takensoft.cms.author.vo.AuthorVO;
import com.takensoft.common.exception.IdDuplicationException;
import com.takensoft.common.util.ResponseData;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author  : takensoft
 * @since   : 2024.04.01
 *
 * 권한 정보 관련 컨트롤러
 */
@RestController
@RequiredArgsConstructor
@Slf4j
@RequestMapping("/admin/auth")
public class AuthorController {

    private final AuthorService authorService;

    /**
     * @author takensoft
     * @since 2024.04.03
     * @param authorVO
     * @return
     * @throws Exception
     *
     * 권한 등록
     */
    @PostMapping("/saveProc.json")
    public ResponseEntity<?> saveProc(@RequestBody AuthorVO authorVO) throws Exception {
        // 권한 코드 중복 검사
        boolean isExistence = authorService.findByCheckAuthorCode(authorVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();

        if(isExistence) {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("이미 존재하는 권한코드입니다.");
            log.error("이미 존재하는 권한코드입니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
        // 권한 등록
        int result = authorService.authorSave(authorVO);

        if(result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 등록 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("등록에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * @param params
     * @return
     * @throws Exception
     *
     * 권한 목록 조회
     */
    @PostMapping("/findAll.json")
    public ResponseEntity<?> findAll(@RequestBody Map<String, String> params) throws Exception {
        // 권한 목록 조회
        Map<String, Object> result = authorService.findAllAuthor(params);
        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * @param authorVO
     * @return
     * @throws Exception
     *
     * 권한 상세 조회
     */
    @PostMapping("/findByAuthrt.json")
    public ResponseEntity<?> findByAuthrt(@RequestBody AuthorVO authorVO) throws Exception {
        // 권한 상세 조회
        AuthorVO authrt = authorService.findByAuthorCode(authorVO.getAuthrtCd());

        Map<String, Object> result = new HashMap<String, Object>();
        result.put("authrt", authrt);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * @param authorVO
     * @return
     * @throws Exception
     *
     * 권한 수정
     */
    @PostMapping("/updateProc.json")
    public ResponseEntity<?> updateProc(@RequestBody AuthorVO authorVO) throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        
        // 권한 코드의 변경 사항이 있을 때만 코드 중복 검사 진행
        if(authorVO.getOriginAuthrtCd() != null && !authorVO.getOriginAuthrtCd().equals("") && !authorVO.getOriginAuthrtCd().equals(authorVO.getAuthrtCd())) {
            // 권한 코드 중복 검사
            boolean isExistence = authorService.findByCheckAuthorCode(authorVO);
            if(isExistence) {
                responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
                responseData.setMessage("이미 존재하는 권한코드입니다.");
                log.error("이미 존재하는 권한코드입니다.");
                return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }
        
        // 권한 수정
        int result = authorService.authorUpdate(authorVO);

        // 응답 처리
        if(result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 수정 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("수정에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * @param authorVO
     * @return
     * @throws Exception
     *
     * 권한 삭제
     */
    @PostMapping("/deleteProc.json")
    public ResponseEntity<?> deleteProc(@RequestBody AuthorVO authorVO) throws Exception {
        // 권한 삭제
        int result = authorService.authorUpdate(authorVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if(result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 삭제 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("삭제에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author 박정하
     * @since 2024.04.26
     * @param
     * @return
     * @throws Exception
     *
     * 전체 권한 목록 조회(신규 메뉴 등록 시 권한을 주기 위해 사용)
     */
    @PostMapping("/findAllSystem.json")
    public ResponseEntity<?> findAllSystem() throws Exception {
        List<AuthorVO> result = authorService.findAllSystem();
        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }
}
