package com.takensoft.cms.bbs.service.Impl;

import com.takensoft.cms.bbs.dao.CmntDAO;
import com.takensoft.cms.bbs.dao.WordMngDAO;
import com.takensoft.cms.bbs.service.CmntService;
import com.takensoft.cms.bbs.vo.CmntVO;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author  : 방선주
 * @since   : 2024.05.27
 *
 * 게시판 댓글 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * CmntService : 게시판 댓글 인터페이스 상속
 */
@Service("cmntService")
@RequiredArgsConstructor
public class CmntServiceImpl extends EgovAbstractServiceImpl implements CmntService {

    private final CmntDAO cmntDAO;
    private final JWTUtil jwtUtil;
    private final WordMngDAO wordMngDAO;

    /**
     * @author 방선주
     * @since  2024.05.27
     *
     * 게시판 댓글 등록
     */
    public HashMap<String, Object> saveCmnt(CmntVO cmntVO) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        List<String> wordList = wordMngDAO.findWordList();

        // 내용에 금지어가 포함되어있는가?
        String content = cmntVO.getCmntCn();
        for (String word : wordList) {
            if (content.contains(word)) {
                result.put("result", -1);
                result.put("word", word);
                return result;
            }
        }

        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        if(writer == null) {
            result.put("result", -2);
            return result;
        }

        cmntVO.setRgtr(writer);

        // 댓글 깊이
        // 부모의 깊이 가져와 +1

        if (cmntVO.getUpCmntId() != 0) {
            CmntVO parentCmnt = cmntDAO.findParentCmnt(cmntVO.getUpCmntId());
            cmntVO.setCmntGrd(parentCmnt.getCmntGrd() + 1);
        }

        // 댓글 순서
        int count = cmntDAO.getCmntCount(cmntVO);
        cmntVO.setCmntSn(count + 1);


        result.put("result", cmntDAO.saveCmnt(cmntVO));

        return result;
    }

    /**
     * @author 방선주
     * @since  2024.05.27
     *
     * 게시판 댓글 목록 조회
     */
    public List<CmntVO> findCmntList(String bbsId) throws Exception {

        List<CmntVO> flatCommentList = cmntDAO.findCmntList(bbsId);
        return buildCommentTree(flatCommentList);
    }

    private List<CmntVO> buildCommentTree(List<CmntVO> comments) {
        Map<Integer, CmntVO> commentMap = new HashMap<>();
        List<CmntVO> rootComments = new ArrayList<>();

        // 댓글 ID를 키로 하여 맵 생성
        for (CmntVO comment : comments) {
            commentMap.put(comment.getCmntId(), comment);
        }

        // 자식 댓글을 부모의 children 리스트에 추가
        for (CmntVO comment : comments) {
            Integer parentId = comment.getUpCmntId();
            if (parentId != null && commentMap.containsKey(parentId)) { // 상위 댓글이 있는 경우
                CmntVO parent = commentMap.get(parentId);
                parent.getChildren().add(comment);
            } else { // 최상위 댓글인 경우 또는 부모가 맵에 없는 경우(데이터 무결성 문제)
                rootComments.add(comment);
            }
        }

        return rootComments; // 최상위 댓글 리스트 반환
    }
    /**
     * @author 방선주
     * @since  2024.05.27
     *
     * 게시판 댓글 업데이트
     */
    public HashMap<String, Object> updateCmnt(CmntVO cmntVO) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        List<String> wordList = wordMngDAO.findWordList();

        // 내용에 금지어가 포함되어있는가?
        String content = cmntVO.getCmntCn();
        for (String word : wordList) {
            if (content.contains(word)) {
                result.put("result", -1);
                result.put("word", word);
                return result;
            }
        }

        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        cmntVO.setMdfr(writer);

        result.put("result", cmntDAO.updateCmnt(cmntVO));

        return result;
    }

    /**
     * @author 방선주
     * @since  2024.05.27
     *
     * 게시판 댓글 삭제
     */
    public int deleteCmnt(CmntVO cmntVO) throws Exception {

        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        cmntVO.setMdfr(writer);

        cmntVO.setUseYn("N");

        return cmntDAO.deleteCmnt(cmntVO);
    }
}
