package com.takensoft.cms.codeManage.service.Impl;

import com.takensoft.cms.codeManage.dao.CodeManageDAO;
import com.takensoft.cms.codeManage.service.CodeManageService;
import com.takensoft.cms.codeManage.vo.CodeManageVO;
import com.takensoft.cms.dept.vo.DeptVO;
import com.takensoft.common.util.JWTUtil;
import com.takensoft.common.HierachyVO;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author  : takensoft
 * @since   : 2024.04.09
 *
 * 공통코드 정보 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * AuthorService : 공통코드 정보 인터페이스 상속
 */
@Service("codeManageService")
@RequiredArgsConstructor
public class CodeManageServiceImpl extends EgovAbstractServiceImpl implements CodeManageService {

    private static final Logger LOGGER = LoggerFactory.getLogger(CodeManageServiceImpl.class);
    private final JWTUtil jwtUtil;
    private final CodeManageDAO codeManageDAO;

    /**
     * @author takensoft
     * @since 2024.04.09
     * 코드 중복 검사
     */
    @Override
    public boolean findByCheckCd(CodeManageVO codeManageVO) throws Exception {
        return codeManageDAO.findByCheckCd(codeManageVO);
    }

    /**
     * @author takensoft
     * @since 2024.04.09
     * 코드 등록
     */
    @Override
    public int cdSave(CodeManageVO codeManageVO) throws Exception {
        // 상위 코드가 존재 하는 경우
        if(codeManageVO.getUpCd() != null && !codeManageVO.getUpCd().equals("")) {
            // 상위 메뉴 아이디 보다 깊이(레벨) 1 추가
            codeManageVO.setCdGrd(codeManageDAO.findByCdGrd(codeManageVO.getUpCd()) + 1);
            // 동일한 깊이의 코드 순서 보다 1 추가
            codeManageVO.setCdSn(codeManageDAO.findByCdSn(codeManageVO.getUpCd()) + 1);
        // 상위 코드가 존재 하지 않는 경우
        } else {
            codeManageVO.setCdGrd(0);
            codeManageVO.setCdSn(codeManageDAO.findByCdSn(null) + 1);
        }
        // 등록된 토큰에서 사용자 정보 조회
        codeManageVO.setRgtr(jwtUtil.getWriter());
        return codeManageDAO.save(codeManageVO);
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * 코드 수정
     */
    @Override
    public int cdUpdate(CodeManageVO codeManageVO) throws Exception {
        // 등록된 토큰에서 사용자 정보 조회
        codeManageVO.setMdfr(jwtUtil.getWriter());
        // 코드 수정
        int result = codeManageDAO.update(codeManageVO);
        // 해당 코드가 변경됨에 따라 하위 코드의 상위코드값 변경
        result += codeManageDAO.updateUpCd(codeManageVO);
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.09
     * 전체 코드 조회 [관리자용 ]
     */
    @Override
    public List<CodeManageVO> findTopCd() throws Exception {
        // 상위 코드 조회
        List<CodeManageVO> topCdList = codeManageDAO.findTopCd();
        // 상위 코드에 따른 하위 코드 조회
        for(CodeManageVO codeManage : topCdList) {
            List<CodeManageVO> childCd = findByChildCd(codeManage.getCd());
            codeManage.setChildCdList(childCd);
        }
        return topCdList;
    }

    /**
     * @author takensoft
     * @since 2024.04.09
     * 하위 코드 조회
     */
    @Override
    public List<CodeManageVO> findByChildCd(String cd) throws Exception {
        List<CodeManageVO> childCdList = codeManageDAO.findChildCd(cd);
        for(CodeManageVO codeManage : childCdList) {
            List<CodeManageVO> childCd = findByChildCd(codeManage.getCd());
            codeManage.setChildCdList(childCd);
        }
        return childCdList;
    }

    /**
     * @author takensoft
     * @since 2024.04.09
     * 코드 상세 조회
     */
    @Override
    public CodeManageVO findByCd(String cd) throws Exception {
        return codeManageDAO.findByCd(cd);
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * 특정 공통 목록 코드 조회(시스템 처리용)
     */
    @Override
    public CodeManageVO findByCdSystem(String cd) throws Exception {
        CodeManageVO codeManage = codeManageDAO.findByCd(cd);
        codeManage.setChildCdList(findByChildCd(cd));
        return codeManage;
    }

    /**
     * @author takensoft
     * @since 2024.04.19
     * 하위 코드 조회(cache)
     */
    @Override
    @Cacheable(value = "findByChildCdCache", key = "#cd")
    public List<CodeManageVO> findByChildCdCache(String cd) throws Exception {
        LOGGER.info("Loading search conditions from the database for boardType: {}", cd);
        return codeManageDAO.findChildCdUser(cd);
    }

    /**
     * @author takensoft
     * @since 2024.04.23
     * 코드 삭제
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int cdDelete(String cd) throws Exception {
        return codeManageDAO.deleteCd(cd);
    }
    /************************************** Hierachy 전용 **************************************/
    /**
     * @author takensoft
     * @since 2024.04.24
     * 최상위 코드 조회
     */
    @Override
    public List<HierachyVO> findByTopNode() {
        // 상위 코드 조회
        List<HierachyVO> topCdList = codeManageDAO.findByTopNode();
        // 상위 코드에 따른 하위 코드 조회
        for(HierachyVO hierachy : topCdList) {
            List<HierachyVO> child = findChildNode(hierachy.getId());
            hierachy.setChildList(child);
        }
        return topCdList;
    }

    /**
     * @author takensoft
     * @since 2024.04.24
     * 하위 코드 조회
     */
    @Override
    public List<HierachyVO> findChildNode(String id) {
        List<HierachyVO> childList = codeManageDAO.findChildNode(id);
        for(HierachyVO hierachyVO : childList) {
            List<HierachyVO> child = findChildNode(hierachyVO.getId());
            hierachyVO.setChildList(child);
        }
        return childList;
    }
    /************************************** Hierachy 전용 **************************************/

    /**
     * @author 박정하
     * @since 2024.05.10
     * 부서 코드 수정 (hierachyVO 사용)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateList(List<HierachyVO> deptList) throws Exception {
        int result = 0;
        String writer = jwtUtil.getWriter();

        if(deptList != null && deptList.size() > 0) {
            int sn = 1;
            for(HierachyVO hierachyVO : deptList) {
                // 최상위 노드 수정
                CodeManageVO vo = new CodeManageVO();
                vo.setCd(hierachyVO.getId());
                vo.setUpCd(null);
                vo.setCdGrd(0);
                vo.setCdSn(sn);
                vo.setMdfr(writer);
                result += codeManageDAO.codeUpdateByHierachy(vo);

                // 하위 노드 수정
                if (hierachyVO.getChildList() != null && hierachyVO.getChildList().size() > 0) {
                    result += updateChildNode(hierachyVO.getChildList(), vo);
                }
                sn++;
            }
        }
        return result;
    }
    /**
     * @author 박정하
     * @since 2024.05.10
     * 하위 코드 수정
     */
    @Override
    public int updateChildNode(List<HierachyVO> childList, CodeManageVO upCodeManageVO) throws Exception {
        int result = 0;
        String upDeptId = upCodeManageVO.getCd();
        int grd = upCodeManageVO.getCdGrd() + 1;
        int sn = 1;
        String mdfr = upCodeManageVO.getMdfr();
        for(HierachyVO hierachyVO : childList) {
            // 현재 노드 수정
            CodeManageVO vo = new CodeManageVO();
            vo.setCd(hierachyVO.getId());
            vo.setUpCd(upDeptId);
            vo.setCdGrd(grd);
            vo.setCdSn(sn);
            vo.setMdfr(mdfr);
            result += codeManageDAO.codeUpdateByHierachy(vo);

            // 하위 노드 수정
            if (hierachyVO.getChildList() != null && hierachyVO.getChildList().size() > 0) {
                result += updateChildNode(hierachyVO.getChildList(), vo);
            }
            sn++;
        }
        return result;
    }
}