package com.takensoft.cms.mber.service.Impl;

import com.takensoft.cms.mber.dao.AdmMbrDAO;
import com.takensoft.cms.mber.dto.AdmMbrDTO;
import com.takensoft.cms.mber.dto.JoinDTO;
import com.takensoft.cms.mber.dto.PasswordDTO;
import com.takensoft.cms.mber.service.AdmMbrService;
import com.takensoft.cms.mber.service.MberService;
import com.takensoft.cms.mber.vo.MberAuthorVO;
import com.takensoft.cms.mber.vo.MberVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.CommonUtils;
import com.takensoft.common.util.JWTUtil;
import com.takensoft.common.util.Secret;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;

/**
 * @author  : 박정하
 * @since   : 2024.06.21
 *
 * 회원정보 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * AdmMbrService : 회원정보 인터페이스 상속
 */
@Service("admMbrService")
@RequiredArgsConstructor
public class AdmMbrServiceImpl extends EgovAbstractServiceImpl implements AdmMbrService {
    private final JWTUtil jwtUtil;
    private final IdgenService mberIdgn;
    private final CommonUtils commonUtils;
    private final BCryptPasswordEncoder bCryptPasswordEncoder;
    private final AdmMbrDAO admMbrDAO;
    private final MberService mbrService;

    /**
     * @author  : 박정하
     * @since   : 2024.06.21
     *
     * 회원정보 목록 조회
     */
    @Override
    public HashMap<String, Object> mbrList(HashMap<String, String> params) throws Exception {
        Pagination search = new Pagination(0, params);
        int cnt = admMbrDAO.mbrListCnt(search);

        Pagination pagination = new Pagination(cnt, params);
        List<AdmMbrDTO> list = admMbrDAO.mbrList(pagination);

        // 휴대폰번호 복호화
        for (AdmMbrDTO admMbrDTO : list) {
            if (admMbrDTO.getMblTelno() != null && !admMbrDTO.getMblTelno().equals("")) {
                admMbrDTO.setMblTelno(Secret.decrypt(admMbrDTO.getMblTelno().toString()));
            }
        }

        HashMap<String, Object> result = new HashMap<>();
        result.put("list", list);
        result.put("pagination", pagination);
        return result;

    }

    /**
     * @author  : 박정하
     * @since   : 2024.06.21
     *
     * 회원정보 상세 조회
     */
    @Override
    public AdmMbrDTO mbrDetail(String mbrId) throws Exception {
        AdmMbrDTO admMbrDTO = new AdmMbrDTO();

        // mbrId가 있는 경우
        if (mbrId != null) {
            admMbrDTO = admMbrDAO.mbrDetail(mbrId);

            // 휴대폰번호 복호화
            if (admMbrDTO.getMblTelno() != null && !admMbrDTO.getMblTelno().equals("")) {
                admMbrDTO.setMblTelno(Secret.decrypt(admMbrDTO.getMblTelno()));
            }

            // 전화번호 복호화
            if (admMbrDTO.getTelno() != null && !admMbrDTO.getTelno().equals("")) {
                admMbrDTO.setTelno(Secret.decrypt(admMbrDTO.getTelno()));
            }
        }
        // mbrId가 없는 경우
        else {
            admMbrDTO.setUseYn("Y");
            admMbrDTO.setMbrStts("1");
            admMbrDTO.setSmsRcptnAgreYn("Y");
            admMbrDTO.setEmlRcptnAgreYn("Y");
        }

        return admMbrDTO;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.06.21
     *
     * 회원정보 수정
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateMbr(AdmMbrDTO admMbrDTO) throws Exception {
        // 비밀번호 변경
        if (admMbrDTO.getPswd() != null && !admMbrDTO.getPswd().equals("")) {
            PasswordDTO passwordDTO = new PasswordDTO();
            passwordDTO.setMbrId(admMbrDTO.getMbrId());
            passwordDTO.setNewPswd(admMbrDTO.getPswd());
            mbrService.updatePassword(passwordDTO);
        }

        // 연락처 암호화
        if(admMbrDTO.getMblTelno() != null && !admMbrDTO.getMblTelno().equals("")){
            admMbrDTO.setMblTelno(Secret.encrypt(admMbrDTO.getMblTelno()));
        }
        if(admMbrDTO.getTelno() != null && !admMbrDTO.getTelno().equals("")) {
            admMbrDTO.setTelno(Secret.encrypt(admMbrDTO.getTelno()));
        }

        // 수정자 조회 및 등록
        String writer = jwtUtil.getWriter();
        admMbrDTO.setMdfr(writer);

        // 회원정보 수정
        int result = admMbrDAO.updateMbr(admMbrDTO);

        // 권한 수정
        result += admMbrDAO.deleteAuthorListByMbrId(admMbrDTO.getMbrId()); // 회원 권한 전체 삭제
        if (admMbrDTO.getUseYn().equals("Y")) {
            if(admMbrDTO.getAuthorList() != null && admMbrDTO.getAuthorList().size() > 0) {
                for(MberAuthorVO mberAuthorVO : admMbrDTO.getAuthorList()) {
                    mberAuthorVO.setMbrId(admMbrDTO.getMbrId());
                    mberAuthorVO.setRgtr(writer);
                    result += admMbrDAO.insertAuthor(mberAuthorVO);
                }
            }
        }

        return result;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.06.21
     *
     * 회원정보 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public HashMap<String, Object> mbrInsert(HttpServletRequest req, JoinDTO joinDTO) throws Exception {
        // 회원 아이디 생성
        String mbrId = mberIdgn.getNextStringId();
        joinDTO.setMbrId(mbrId);

        // 비밀번호 암호화
        joinDTO.setPswd(bCryptPasswordEncoder.encode(joinDTO.getPswd()));

        // 연락처 암호화
        if(joinDTO.getMblTelno() != null && !joinDTO.getMblTelno().equals("")) {
            joinDTO.setMblTelno(Secret.encrypt(joinDTO.getMblTelno()));
        }
        if(joinDTO.getTelno() != null && !joinDTO.getTelno().equals("")) {
            joinDTO.setTelno(Secret.encrypt(joinDTO.getTelno()));
        }

        // 아이피 등록
        String ip = commonUtils.getIp(req);
        joinDTO.setFrstRegIp(ip);

        // 작성자 등록
        String writer = jwtUtil.getWriter();
        if(writer != null && !writer.equals("")) {
            joinDTO.setRgtr(writer);
        }

        // 회원정보 등록
        int insertResult = admMbrDAO.mbrInsert(joinDTO);

        // 권한 등록
        if(joinDTO.getAuthorList().size() > 0) {
            for(MberAuthorVO mberAuthorVO : joinDTO.getAuthorList()) {
                mberAuthorVO.setMbrId(joinDTO.getMbrId());
                mberAuthorVO.setRgtr(writer);
                insertResult += admMbrDAO.insertAuthor(mberAuthorVO);
            }
        }

        HashMap<String, Object> result = new HashMap<>();
        result.put("mbrId", mbrId);
        result.put("insertResult", insertResult);
        return result;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.07.03
     *
     * 아이디 찾기 (로그인 아이디)
     */
    public String lgnIdSearch(MberVO mbrVO) throws Exception {
        return admMbrDAO.lgnIdSearch(mbrVO);
    }

    /**
     * @author  : 박정하
     * @since   : 2024.07.03
     *
     * 아이디 찾기 (멤버 아이디)
     */
    public int mbrIdSearch(String resetPswd, AdmMbrDTO admMbrDTO) throws Exception {
        String mbrId = admMbrDAO.mbrIdSearch(admMbrDTO);

        PasswordDTO passwordDTO = new PasswordDTO();
        passwordDTO.setMbrId(mbrId);
        passwordDTO.setPswd(admMbrDTO.getPswd());
        passwordDTO.setNewPswd(resetPswd);

        // 비밀번호 변경
        int result = mbrService.updatePassword(passwordDTO);

        return result;
    }
}