package com.takensoft.cms.mber.service.Impl;

import com.takensoft.cms.mber.dao.MberDAO;
import com.takensoft.cms.mber.dto.JoinDTO;
import com.takensoft.cms.mber.dto.PasswordDTO;
import com.takensoft.cms.mber.service.MberService;
import com.takensoft.cms.mber.vo.MberAuthorVO;
import com.takensoft.cms.mber.vo.MberVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.CommonUtils;
import com.takensoft.common.util.JWTUtil;
import com.takensoft.common.util.Secret;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;

/**
 * @author  : takensoft
 * @since   : 2024.04.01
 *
 * 회원 정보 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * UserDetailsService : security 사용을 위해 상속
 * MberService : 회원정보 인터페이스 상속
 */
@Service("mberService")
@RequiredArgsConstructor
public class MberServiceImpl extends EgovAbstractServiceImpl implements UserDetailsService, MberService {
    private final MberDAO mberDAO;
    private final IdgenService mberIdgn;
    private final BCryptPasswordEncoder bCryptPasswordEncoder;
    private final JWTUtil jwtUtil;
    private final CommonUtils commonUtils;

    /**
     * @author takensoft
     * @since 2024.04.03
     *
     * security 상속 시 Override 하는 메소드
     */
    @Override
    @Transactional(readOnly = true)
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        return mberDAO.findByMberSecurity(username);
    }

    /**
     * @author takensoft
     * @since 2024.04.03
     *
     * 아이디 중복 검사
     */
    @Override
    public boolean findByCheckLoginId(String lgnId) throws Exception {
        return mberDAO.findByCheckLoginId(lgnId);
    }

    /**
     * @author takensoft
     * @since 2024.04.03
     * 회원가입
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public HashMap<String, Object> userJoin(HttpServletRequest req,  JoinDTO joinDTO) throws Exception {
        // 회원 아이디 생성
        String mbrId = mberIdgn.getNextStringId();
        joinDTO.setMbrId(mbrId);

        // 비밀번호 암호화
        joinDTO.setPswd(bCryptPasswordEncoder.encode(joinDTO.getPswd()));

        // 연락처 암호화
        if(joinDTO.getMblTelno() != null && !joinDTO.getMblTelno().equals("")) {
            joinDTO.setMblTelno(Secret.encrypt(joinDTO.getMblTelno()));
        }
        if(joinDTO.getTelno() != null && !joinDTO.getTelno().equals("")) {
            joinDTO.setTelno(Secret.encrypt(joinDTO.getTelno()));
        }

        // 아이피 조회 및 등록
        joinDTO.setFrstRegIp(commonUtils.getIp(req));
        
        // 작성자 조회 및 등록
        if(jwtUtil.getWriter() != null && !jwtUtil.getWriter().equals("")) {
            joinDTO.setRgtr(jwtUtil.getWriter());
        }

        // 회원정보 등록
        HashMap<String, Object> result = new HashMap<>();
        int saveResult = mberDAO.save(joinDTO);
        result.put("mbrId", mbrId);

        // 권한 등록
        int authorResult = 0;
        if(joinDTO.getAuthorList().size() > 0) {
            for(MberAuthorVO vo : joinDTO.getAuthorList()) {
                vo.setMbrId(joinDTO.getMbrId());
                // 작성자 조회 및 등록
                if(jwtUtil.getWriter() != null && !jwtUtil.getWriter().equals("")) {
                    vo.setRgtr(jwtUtil.getWriter());
                }
                authorResult += mberDAO.authorSave(vo);
            }
        }
        result.put("result", saveResult + authorResult);

        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * 비밀번호 비교
     */
    @Override
    public boolean passwordCheck(PasswordDTO passwordDTO) throws Exception {
        // 회원 정보 호출
        MberVO mbr = mberDAO.findByMber(jwtUtil.getWriter());
        // 비밀번호 비교 후 성공 시 비밀번호 수정 후 true 반환
        if(bCryptPasswordEncoder.matches(passwordDTO.getPswd(), mbr.getPassword())) {
            passwordDTO.setNewPswd(bCryptPasswordEncoder.encode(passwordDTO.getNewPswd()));
            passwordDTO.setMbrId(mbr.getMbrId());
            mberDAO.updatePassword(passwordDTO);
            return true;
            // 기존 비밀번호와 입력한 비밀번호가 서로 다를 경우 false 반환
        } else {
            return false;
        }
    }

    /**
     * @author 박정하
     * @since 2024.04.23
     * 회원정보 상세 조회
     */
    @Override
    public MberVO findByMbr(HashMap<String, Object> params) throws Exception {
        String mbrId = params.get("mbrId").toString();
        MberVO mberVO = mberDAO.findByMber(mbrId);
        // 휴대폰번호 복호화
        if (mberVO.getMblTelno() != null && !mberVO.getMblTelno().equals("")) {
            mberVO.setMblTelno(Secret.decrypt(mberVO.getMblTelno().toString()));
        }
        // 전화번호 복호화
        if (mberVO.getTelno() != null && !mberVO.getTelno().equals("")) {
            mberVO.setTelno(Secret.decrypt(mberVO.getTelno().toString()));
        }
        return mberVO;
    }

    /**
     * @author takensoft
     * @since 2024.04.24
     * 비밀번호 수정
     */
    @Override
    public int updatePassword(PasswordDTO passwordDTO) throws Exception {
        passwordDTO.setNewPswd(bCryptPasswordEncoder.encode(passwordDTO.getNewPswd()));
        return mberDAO.updatePassword(passwordDTO);
    }
}