package com.takensoft.cms.mber.service.Impl;

import com.fasterxml.jackson.datatype.jsr310.ser.YearSerializer;
import com.takensoft.cms.mber.dao.RefreshTokenDAO;
import com.takensoft.cms.mber.service.RefreshTokenService;
import com.takensoft.cms.mber.vo.MberAuthorVO;
import com.takensoft.cms.mber.vo.MberVO;
import com.takensoft.cms.mber.vo.RefreshVO;
import com.takensoft.common.util.CommonUtils;
import com.takensoft.common.util.JWTUtil;
import io.jsonwebtoken.ExpiredJwtException;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author  : takensoft
 * @since   : 2024.04.01
 *
 * RefreshToken 정보 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * RefreshTokenService : RefreshToken 인터페이스 상속
 */
@Service("refreshTokenService")
@RequiredArgsConstructor
public class RefreshTokenServiceImpl extends EgovAbstractServiceImpl implements RefreshTokenService {

    private static final Logger LOGGER = LoggerFactory.getLogger(RefreshTokenServiceImpl.class);
    private final RefreshTokenDAO refreshTokenDAO;
    private final JWTUtil jwtUtil;
    private final CommonUtils commonUtils;

    @Value("${jwt.accessTime}")
    private long JWT_ACCESSTIME; // access 토큰 유지 시간
    @Value("${jwt.refreshTime}")
    private long JWT_REFRESHTIME; // refresh 토큰 유지 시간
    @Value("${cookie.time}")
    private int COOKIE_TIME; // 쿠키 유지 시간

    /**
     * @author takensoft
     * @since 2024.04.04
     * refresh token 검증
     */
    private Map<String, Object> refreshTokenCheck(HttpServletRequest req) {
        Map<String, Object> result = new HashMap <String, Object>();
        // header 방식
//        String refreshToken = req.getHeader("refresh");
        // 쿠키 방식
        String refreshToken = null;
        Cookie[] cookies = req.getCookies();
        for (Cookie cookie : cookies) {
            if (cookie.getName().equals("refresh")) refreshToken = cookie.getValue();
        }
        if(refreshToken == null) {
            result.put("result", 0);
            return result;
        }
        try {
            jwtUtil.isExpired(refreshToken);
        } catch (ExpiredJwtException e) {
            result.put("result", 0);
            return result;
        }
        String category = jwtUtil.getCategory(refreshToken);
        if(!category.equals("refresh")) {
            result.put("result", 0);
            return result;
        }
        result.put("result", 1);
        result.put("refreshToken", refreshToken);

        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.04
     * refresh token 등록
     */
    @Override
    public int saveRefreshToken(HttpServletRequest req, HttpServletResponse res, RefreshVO refresh, long expiredMs) throws Exception {
        // 만료 시간
        refresh.setExpryDt(new Date(System.currentTimeMillis() + expiredMs));
        // 사용중인 아이피
        refresh.setUseIp(commonUtils.getIp(req));
        return refreshTokenDAO.save(refresh);
    }

    /**
     * @author takensoft
     * @since 2024.04.04
     * refresh token 삭제 프로세스 (로그아웃)
     */
    @Override
    public int deleteByRefresh(HttpServletRequest req, HttpServletResponse res) throws Exception {
        // refresh token 검증
        int result = (int) refreshTokenCheck(req).get("result");
        if(result == 0) return result;

        //Refresh 토큰 Cookie 값 0
//        res.addCookie(jwtUtil.createCookie("refresh",null, 0));

        RefreshVO refreshVO = new RefreshVO();
        refreshVO.setMbrId(jwtUtil.getMbrId(refreshTokenCheck(req).get("refreshToken").toString()));

        return delete(req, refreshVO);
    }

    /**
     * @author takensoft
     * @since 2024.04.04
     * 토큰 재발급
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int tokenReissueProc(HttpServletRequest req, HttpServletResponse res) throws Exception {
        // refresh token 검증
        int result = (int) refreshTokenCheck(req).get("result");
        if(result == 0) return result;

        String refreshToken = refreshTokenCheck(req).get("refreshToken").toString();

        Date expired = jwtUtil.getExpired(refreshToken);

        // 만료시간과 현재 시간의 차이 계산
        long timeDffrnc = (expired.getTime() - new Date().getTime()) / (1000 * 60 * 60);

        MberVO mber = new MberVO();
        List<MberAuthorVO> roles = jwtUtil.getRoles(refreshToken);
        mber.setLgnId(jwtUtil.getLgnId(refreshToken));
        mber.setMbrId(jwtUtil.getMbrId(refreshToken));
        mber.setMbrNm(jwtUtil.getMbrNm(refreshToken));
        mber.setAuthorList(roles);
        // 신규 AccessToken 발행
        String newAccessToken = jwtUtil.createJwt("Authorization", mber.getMbrId(), mber.getLgnId(), mber.getMbrNm(), (List) mber.getAuthorities(), JWT_ACCESSTIME);

        // 남은 시간이 3시간 미만으로 남았을 경우 RefreshToken 재발급
        if(timeDffrnc < 3) {
            RefreshVO refresh = new RefreshVO();
            refresh.setMbrId(mber.getMbrId());

            // 기존 RefreshToken 삭제
            result += delete(req, refresh);

            // 신규 RefreshToken 발행
            String newRefreshToken = jwtUtil.createJwt("refresh", mber.getMbrId(), mber.getLgnId(), mber.getMbrNm(), (List) mber.getAuthorities(), JWT_REFRESHTIME);
            refresh.setToken(newRefreshToken);
            result += saveRefreshToken(req, res,refresh, JWT_REFRESHTIME);

            // 응답설정 RefreshToken
//            res.setHeader("refresh", newRefreshToken);
            // 쿠키 방식
            res.addCookie(jwtUtil.createCookie("refresh",newRefreshToken, COOKIE_TIME));
        }
        // 응답설정 AccessToken
        res.setHeader("Authorization", newAccessToken);

        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.04
     * 토큰 삭제
     */
    @Override
    public int delete(HttpServletRequest req, RefreshVO refreshVO) throws Exception {
        refreshVO.setUseIp(commonUtils.getIp(req));
        return refreshTokenDAO.deleteByRefresh(refreshVO);
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * refresh token 등록 유무 확인
     */
    @Override
    public boolean findByCheckRefresh(HttpServletRequest req, RefreshVO refreshVO) throws Exception {
        // 사용중인 아이피
        refreshVO.setUseIp(commonUtils.getIp(req));
        return refreshTokenDAO.findByCheckRefresh(refreshVO);
    }

}
