package com.takensoft.cms.menu.service.Impl;

import com.takensoft.cms.menu.dao.MenuAuthorDAO;
import com.takensoft.cms.menu.dao.MenuDAO;
import com.takensoft.cms.menu.service.MenuAuthorService;
import com.takensoft.cms.menu.service.MenuService;
import com.takensoft.cms.menu.vo.MenuAuthorVO;
import com.takensoft.cms.menu.vo.MenuRequest;
import com.takensoft.cms.menu.vo.MenuVO;
import com.takensoft.cms.menu.vo.SysMenuVO;
import com.takensoft.common.HierachyVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author  : takensoft
 * @since   : 2024.04.01
 *
 * 메뉴 정보 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * AuthorService : 메뉴 정보 인터페이스 상속
 */
@Service("menuServiceImpl")
@RequiredArgsConstructor
public class MenuServiceImpl extends EgovAbstractServiceImpl implements MenuService {
    private final MenuDAO menuDAO;
    private final MenuAuthorService menuAuthorService;
    private final MenuAuthorDAO menuAuthorDAO;
    private final IdgenService menuIdgn;
    private final JWTUtil jwtUtil;

    /**
     * @author takensoft
     * @since 2024.04.15
     * 메뉴별 권한 등록을 위한 전처리
     */
    private MenuAuthorVO saveMenuAuthor(String menuId, String AuthrtCd) {
        MenuAuthorVO menuAuthorVO = new MenuAuthorVO();
        menuAuthorVO.setMenuId(menuId);
        menuAuthorVO.setAuthrtCd(AuthrtCd);
        return menuAuthorVO;
    }

    /**
     * @author takensoft
     * @since 2024.04.08
     * 메뉴 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = "findByMenuWithRouter", allEntries = true)
    public int menuSave(MenuVO menuVO) throws Exception {
        // 메뉴 아이디 생성
        String menuId = menuIdgn.getNextStringId();
        menuVO.setMenuId(menuId);
        // 상위 메뉴가 존재하는 경우
        if(menuVO.getUpMenuId() != null && !menuVO.getUpMenuId().equals("")) {
            // 상위 메뉴 아이디 보다 깊이(레벨) 1추가
            menuVO.setMenuGrd(menuDAO.findByMenuGrd(menuVO.getUpMenuId()) + 1);
            // 동일한 깊이의 메뉴 순서 보다 1추가
            menuVO.setMenuSn(menuDAO.findByMenuSn(menuVO.getUpMenuId()) + 1);
        // 상위메뉴가 존재 하지 않는 경우
        } else {
            menuVO.setMenuGrd(0);
            menuVO.setMenuSn(menuDAO.findByMenuSn(null) + 1);
        }
        // 작성자 정보
        String writer = jwtUtil.getWriter();
        menuVO.setRgtr(writer);
        // 시스템 제공 여부
        menuVO.setSysPvsnYn("1");
        if(menuVO.getMenuGrd() != 0) {
            String upMenuId = menuVO.getUpMenuId();
            for (int i = 0; i < menuVO.getMenuGrd() - 1; i++) {
                upMenuId = menuDAO.findUpMenuId(upMenuId);
            }
            if (upMenuId.equals("MENU_000000000000001")) {
                menuVO.setSysPvsnYn("0");
            }
        }
        // 메뉴 등록
        int result = menuDAO.save(menuVO);

        // 메뉴별 권한 등록
        if(result > 0 && menuVO.getMenuGrd() != 0) {
            String oriMenuId = menuVO.getMenuId();
            for (int i = 0; i < menuVO.getMenuGrd(); i++) {
                oriMenuId = menuDAO.findUpMenuId(oriMenuId);
            }
            if (oriMenuId.equals("MENU_000000000000001")) {
                MenuAuthorVO menuAuthor = saveMenuAuthor(menuId, "ROLE_ADMIN");
                result += menuAuthorDAO.menuAuthrtSave(menuAuthor);
            } else {
                result += menuAuthorService.menuAuthrtSave(menuId, "menu");
            }
        }
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.08
     * 전체 메뉴 조회
     * !!현재 java버전과 mybatis버전이 낮은관계로 재귀적 쿼리 작성 불가!!
     */
    @Override
    public List<MenuVO> findAll() throws Exception {
        // 상위 메뉴 조회
        List<MenuVO> topMenus = menuDAO.findTopMenu();
        // 상위 메뉴에 따른 하위 메뉴 조회
        for(MenuVO menu : topMenus) {
            List<MenuVO> childMenus = findByChildMenu(menu.getMenuId());
            menu.setChildMenuList(childMenus);
        }

        return topMenus;
    }

    /**
     * @author takensoft
     * @since 2024.04.08
     * 하위 메뉴 조회
     */
    @Override
    public List<MenuVO> findByChildMenu(String menuId) throws Exception {
        List<MenuVO> childMenus = menuDAO.findChildMenus(menuId);
        for(MenuVO menu : childMenus) {
            List<MenuVO> grandChildMenus = findByChildMenu(menu.getMenuId());
            menu.setChildMenuList(grandChildMenus);
        }
        return childMenus;
    }

    /**
     * @author takensoft
     * @since 2024.04.08
     * 메뉴 상세 조회
     */
    @Override
    public MenuVO findByMenu(String menuId) throws Exception {
        return menuDAO.findByMenu(menuId);
    }

    /**
     * @author takensoft
     * @since 2024.04.09
     * 메뉴 수정
     */
    @Override
    @CacheEvict(value = "findByMenuWithRouter", allEntries = true)
    public int MenuUpdate(MenuVO menuVO) throws Exception {
        // 토큰정보에서 사용자 아이디 조회 후 등록
        menuVO.setMdfr(jwtUtil.getWriter());
        return menuDAO.update(menuVO);
    }

    /************************************** Hierachy 전용 **************************************/
    /**
     * @author takensoft
     * @since 2024.04.25
     * 최상위 메뉴 조회
     */
    @Override
    public List<HierachyVO> findByTopNode() throws Exception {
        List<HierachyVO> topDeptList = menuDAO.findByTopNode();
        for(HierachyVO vo : topDeptList) {
            List<HierachyVO> child = findChildNode(vo.getId());
            vo.setChildList(child);
        }
        return topDeptList;
    }

    /**
     * @author takensoft
     * @since 2024.04.25
     * 하위 메뉴 조회
     */
    @Override
    public List<HierachyVO> findChildNode(String id) throws Exception {
        List<HierachyVO> childList = menuDAO.findChildNode(id);
        for(HierachyVO vo : childList) {
            List<HierachyVO> child = findChildNode(vo.getId());
            vo.setChildList(child);
        }
        return childList;
    }
    /************************************** Hierachy 전용 **************************************/

    /**
     * @author 박정하
     * @since 2024.04.29
     * 메뉴 정보 삭제
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int menuDelete(MenuVO menuVO) throws Exception {
        menuVO.setMdfr(jwtUtil.getWriter());
        return menuDAO.deleteMenu(menuVO);
    }

    /************************************** 홈페이지 메뉴 호출 전용 **************************************/
    /**
     * @author takensoft
     * @since 2024.05.09
     * 메뉴 조회(시스템용)
     */
    @Override
    @Cacheable(value = "findByMenuWithRouter", key = "#mReq.menuType")
    public List<SysMenuVO> findByMenuWithRouter(MenuRequest mReq) throws Exception {
        if(mReq.isChck() && mReq.getMenuType().equals("adm")) {
            mReq.setMenuId("MENU_000000000000001");
        } else if(mReq.isChck() && mReq.getMenuType().equals("company")) {
            mReq.setMenuId("MENU_000000000000002");
        } else if(mReq.isChck() && mReq.getMenuType().equals("government")) {
            mReq.setMenuId("MENU_000000000000003");
        }
        mReq.setChck(false);
        List<SysMenuVO> result = menuDAO.findByMenuWithRouter(mReq);
        for(SysMenuVO vo : result) {
            mReq.setMenuId(vo.getMenuId());
            List<SysMenuVO> child = findByMenuWithRouter(mReq);
            vo.setChildList(child);
        }
        return result;
    }
    /************************************** 홈페이지 메뉴 호출 전용 **************************************/

    /**
     * @author 박정하
     * @since 2024.05.10
     * 메뉴 정보 수정 (hierachyVO 사용)
     */
    @Override
    @CacheEvict(value = "findByMenuWithRouter", allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    public int updateList(List<HierachyVO> deptList) throws Exception {
        int result = 0;
        String writer = jwtUtil.getWriter();

        if(deptList != null && deptList.size() > 0) {
            int sn = 1;
            for(HierachyVO hierachyVO : deptList) {
                // 최상위 노드 수정
                MenuVO vo = new MenuVO();
                vo.setMenuId(hierachyVO.getId());
                vo.setUpMenuId(null);
                vo.setMenuGrd(0);
                vo.setMenuSn(sn);
                vo.setMdfr(writer);
                result += menuDAO.menuUpdateByHierachy(vo);

                // 하위 노드 수정
                if (hierachyVO.getChildList() != null && hierachyVO.getChildList().size() > 0) {
                    result += updateChildNode(hierachyVO.getChildList(), vo);
                }
                sn++;
            }
        }
        return result;
    }
    /**
     * @author 박정하
     * @since 2024.05.10
     * 하위 메뉴 수정
     */
    @Override
    @CacheEvict(value = "findByMenuWithRouter", allEntries = true)
    public int updateChildNode(List<HierachyVO> childList, MenuVO upMenuVO) throws Exception {
        int result = 0;
        String upDeptId = upMenuVO.getMenuId();
        int grd = upMenuVO.getMenuGrd() + 1;
        int sn = 1;
        String mdfr = upMenuVO.getMdfr();
        for(HierachyVO hierachyVO : childList) {
            // 현재 노드 수정
            MenuVO vo = new MenuVO();
            vo.setMenuId(hierachyVO.getId());
            vo.setUpMenuId(upDeptId);
            vo.setMenuGrd(grd);
            vo.setMenuSn(sn);
            vo.setMdfr(mdfr);
            result += menuDAO.menuUpdateByHierachy(vo);

            // 하위 노드 수정
            if (hierachyVO.getChildList() != null && hierachyVO.getChildList().size() > 0) {
                result += updateChildNode(hierachyVO.getChildList(), vo);
            }
            sn++;
        }
        return result;
    }
    /**
     * @author 방선주
     * @since 2024.06.07
     * 게시판 관리에서 게시판 삭제 시 게시판 메뉴 미노출로 변경
     */
    @Override
    @CacheEvict(value = "findByMenuWithRouter", allEntries = true)
    public int bbsMngMenuDelete(MenuVO menuVO) throws Exception {
        return menuDAO.bbsMngMenuDelete(menuVO);
    }
}
