package com.takensoft.cms.popup.service.Impl;

import com.takensoft.cms.codeManage.service.CodeManageService;
import com.takensoft.cms.codeManage.vo.CodeManageVO;
import com.takensoft.cms.popup.dao.PopupDAO;
import com.takensoft.cms.popup.service.PopupService;
import com.takensoft.cms.popup.vo.PopupVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.file.service.FileMngService;
import com.takensoft.common.file.vo.FileMngVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.05.13
 *
 * 팝업 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * PopupService : 팝업 관련 인터페이스 상속
 */
@Service("popupService")
@RequiredArgsConstructor
public class PopupServiceImpl extends EgovAbstractServiceImpl implements PopupService {
    private final JWTUtil jwtUtil;
    private final PopupDAO popupDAO;
    private final IdgenService popupIdgn;
    private final FileMngService fileMngService;
    private final CodeManageService codeManageService;

    /**
     * @author  : 박정하
     * @since   : 2024.05.13
     *
     * 팝업 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public HashMap<String, Object> popupInsert(PopupVO popupVO, List<MultipartFile> multipartFileList) throws Exception {
        // 투자상담 아이디 생성
        String popupId = popupIdgn.getNextStringId();
        popupVO.setPopupId(popupId);

        // 등록자 삽입
        String writer = jwtUtil.getWriter();
        popupVO.setRgtr(writer);

        HashMap<String, Object> result = new HashMap<>();

        // 첨부파일 유무
        if (multipartFileList != null && !multipartFileList.isEmpty()) {
            FileMngVO fileMngVO = new FileMngVO();

            // 파일매니저 등록
            HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, multipartFileList);
            result.put("multipartFileMngResult", fileMng);

            // 파일매니저 아이디 삽입
            popupVO.setFileMngId(fileMng.get("fileMngId").toString());
        }

        // 팝업 등록
        int insertResult = popupDAO.insertPopup(popupVO);

        result.put("insertResult", insertResult);
        result.put("popupId", popupVO.getPopupId());

        return result;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.05.13
     *
     * 팝업 목록 조회
     */
    @Override
    public HashMap<String, Object> popupList(HashMap<String, String> params) throws Exception {
        Pagination search = new Pagination(0, params);
        int cnt = popupDAO.selectPopupListCnt(search);

        Pagination pagination = new Pagination(cnt, params);
        List<PopupVO> list = popupDAO.selectPopupList(pagination);

        HashMap<String, Object> result = new HashMap<>();
        result.put("list", list);
        result.put("pagination", pagination);
        return result;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.05.13
     *
     * 팝업 상세 조회
     */
    @Override
    public HashMap<String, Object> popupDetail(PopupVO popupVO) throws Exception {
        PopupVO newPopupVO = new PopupVO();
        if (popupVO.getPopupId() != null) {
            newPopupVO = popupDAO.selectPopupDetail(popupVO);
        }

        // 팝업크기 설정
        if (newPopupVO.getWdthLen() != 0 && newPopupVO.getVrtcLen() != 0) {
            if (newPopupVO.getWdthLen() == 300 && newPopupVO.getVrtcLen() == 300) {
                newPopupVO.setPopupSizeType("fixed");
                newPopupVO.setPopupSize("PS1");
            } else if (newPopupVO.getWdthLen() == 500 && newPopupVO.getVrtcLen() == 700) {
                newPopupVO.setPopupSizeType("fixed");
                newPopupVO.setPopupSize("PS2");
            } else if (newPopupVO.getWdthLen() == 700 && newPopupVO.getVrtcLen() == 1000) {
                newPopupVO.setPopupSizeType("fixed");
                newPopupVO.setPopupSize("PS3");
            } else if (newPopupVO.getWdthLen() == 1200 && newPopupVO.getVrtcLen() == 850) {
                newPopupVO.setPopupSizeType("fixed");
                newPopupVO.setPopupSize("PS4");
            } else {
                newPopupVO.setPopupSizeType("custom");
            }
        } else {
            newPopupVO.setPopupSizeType("fixed");
            newPopupVO.setPopupSize("PS1");
        }

        // 업로드 형식 설정
        if (newPopupVO.getPopupType() == null) {
            newPopupVO.setPopupType("image");
        }

        // 팝업 노출 페이지
        if (newPopupVO.getPageType() == null) {
            newPopupVO.setPageType("");
        }

        // 팝업 사용 유무
        if (newPopupVO.getPopupUseYn() == null) {
            newPopupVO.setPopupUseYn("Y");
        }

        // 사용 유무
        if (newPopupVO.getUseYn() == null) {
            newPopupVO.setUseYn("Y");
        }

        List<CodeManageVO> codeList = codeManageService.findByChildCdCache("pageType"); // 검색 조건

        HashMap<String, Object> result = new HashMap<>();
        result.put("popupVO", newPopupVO);
        result.put("codeList", codeList);

        return result;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.05.14
     *
     * 팝업 수정
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int popupUpdate(PopupVO popupVO, List<MultipartFile> multipartFileList) throws Exception {
        // 수정자 삽입
        String writer = jwtUtil.getWriter();
        popupVO.setMdfr(writer);

        // 파일 수정
        int fileUpdateResult = 0;
        FileMngVO fileMngVO = new FileMngVO();
        if (popupVO.getFileMngId() != null) {
            List<HashMap<String, Object>> deleteFileList = new ArrayList<>();
            if (popupVO.getDeleteFileList() != null && popupVO.getDeleteFileList().size() > 0) {
                deleteFileList = popupVO.getDeleteFileList();
            }

            // 파일매니저 수정
            fileMngVO.setFileMngId(popupVO.getFileMngId());
            fileUpdateResult += fileMngService.fileMngUpadate(fileMngVO, deleteFileList, multipartFileList);
        } else {
            // 파일매니저 등록
            if(multipartFileList != null && multipartFileList.size() > 0) {
                HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, multipartFileList);
                fileUpdateResult += (int) fileMng.get("fileMngResult");
                popupVO.setFileMngId(fileMng.get("fileMngId").toString());
            }
        }

        // 팝업 수정
        int updateResult = popupDAO.updatePopup(popupVO);

        return updateResult + fileUpdateResult;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.05.14
     *
     * 팝업 삭제
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int popupDelete(PopupVO popupVO) throws Exception {
        // 수정자 삽입
        String writer = jwtUtil.getWriter();
        popupVO.setMdfr(writer);

        // 파일매니저 삭제
        int fileDeleteResult = 0;
        if (popupVO.getFileMngId() != null && popupVO.getFileMngId().equals("")) {
            HashMap<String, Object> hashMap = new HashMap<>();
            hashMap.put("fileMngId", popupVO.getFileMngId());
            fileDeleteResult = fileMngService.fileMngDelete(hashMap);
        }

        // 팝업 삭제
        popupVO.setUseYn("N");
        popupVO.setFileMngId(null);
        int deleteResult = popupDAO.updatePopup(popupVO);

        return deleteResult + fileDeleteResult;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.05.22
     *
     * 팝업 목록 조회 (팝업창 띄우는 용도)
     */
    @Override
    public List<PopupVO> popupListByPage(String pageType) throws Exception {
        return popupDAO.selectPopupListByPage(pageType);
    }
}