package com.takensoft.common.file.service.Impl;

import com.takensoft.common.file.dao.FileDAO;
import com.takensoft.common.file.service.FileService;
import com.takensoft.common.file.vo.FileVO;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.UUID;

@Service("fileService")
@RequiredArgsConstructor
public class FileServiceImpl extends EgovAbstractServiceImpl implements FileService {

    private final FileDAO fileDAO;
    private final JWTUtil jwtUtil;

    @Value("${front.url}")
    private String frontUrl;

    @Value("${server.port}")
    private String serverPort;

    @Value("${spring.web.resources.static-locations}")
    private String locationPath;

    @Value("${file.file-upload-path}")
    private String fileUploadPath;

    @Value("${file.edit-file-upload-path}")
    private String editPath;

    /**
     * @author 박정하
     * @since 2024.03.26
     * <p>
     * 파일 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int fileInsert(String fileMngId, List<MultipartFile> multipartFileList) throws Exception {
        int result = 0;

        for (MultipartFile file : multipartFileList) {
            if (file.isEmpty()) {
                continue;
            }

            int size = Long.valueOf(file.getSize()).intValue() / 100000;
            if (size > 200) {
                return -1;
            }

            // 절대 경로 생성
            // "file:///" 제거하여 실제 파일 시스템 경로만을 저장
            String uploadPath = null;
//            String[] pathArray = locationPath.split("/, ");
//            for (String path : pathArray) {
//                if (path.startsWith("file:///")) {
//                    // "file:///" 부분을 제거하고 경로를 반환
//                    uploadPath = path.replace("file:///", "") + fileUploadPath;
//                }
//            }
            uploadPath = locationPath + fileUploadPath;

            // 파일 정보 가공
            String fileName = file.getOriginalFilename(); // 파일명 (원본)
            String fileNm = fileName.substring(0, fileName.lastIndexOf(".")); // 파일명
            String maskNm = UUID.randomUUID().toString().replaceAll("-", ""); // 마스크명
            String extsn = StringUtils.getFilenameExtension(fileName); // 확장자
            String mg = Long.toString(file.getSize()); // 크기
            String absltPath = makeDirectories(uploadPath) + File.separator + maskNm + "." + extsn; // 절대경로
            String partPath = File.separator + maskNm + "." + extsn; // 일부경로

            // FileVO 생성
            FileVO fileVO = new FileVO();
            fileVO.setFileMngId(fileMngId); // 파일관리아이디
            fileVO.setFileNm(fileNm); // 파일명
            fileVO.setMaskNm(maskNm); // 마스크명
            // 파일타입 추후에 추가
            fileVO.setAbsltPath(absltPath); // 절대경로
            fileVO.setPartPath(partPath); // 일부경로
            fileVO.setExtnNm(extsn); // 확장자
            fileVO.setFileSz(mg); // 크기
            // 등록자 생성
            String writer = jwtUtil.getWriter();
            fileVO.setRgtr(writer);

            try {
                try {
                    // 파일 업로드
                    File uploadFile = new File(absltPath);
                    file.transferTo(uploadFile);
                } catch (IOException e) {
                    e.printStackTrace();
                    return 0;
                }

                // DB등록
                result += fileDAO.fileInsert(fileVO);
            } catch (IOException e) {
                e.printStackTrace();
                return 0;
            }
        }

        return result;
    }

    // 업로드 폴더(디렉터리) 생성
    private String makeDirectories(final String path) {
        File dir = new File(path);
        if (dir.exists() == false) {
            dir.mkdirs();
        }
        return dir.getPath();
    }

    /**
     * @author 박정하
     * @since 2024.03.26
     * <p>
     * 파일 목록 조회
     */
    @Override
    public List<HashMap<String, Object>> fileSelectList(String fileMngId) throws Exception {
        return fileDAO.fileSelectList(fileMngId);
    }

    /**
     * @author 박정하
     * @since 2024.03.27
     * <p>
     * 파일 삭제
     */
    public int fileDelete(List<HashMap<String, Object>> fileList) throws Exception {
        int result = 0;

        for (HashMap<String, Object> file : fileList) {
            if (file.isEmpty()) {
                continue;
            }

            try {
                // DB 삭제
                result += fileDAO.fileDelete(file);

                // http 부분
                String url = frontUrl + ":" + serverPort;
                String uploadPath = null;
//                String[] pathArray = locationPath.split("/, ");
//                for (String path : pathArray) {
//                    if (path.startsWith("file:///")) {
//                        // "file:///" 부분을 제거하고 경로를 반환
//                        uploadPath = path.replace("file:///", "") + fileUploadPath;
//                    }
//                }
                uploadPath = locationPath + fileUploadPath;

                String path = file.get("absltPath").toString();
                path = path.replace(url, uploadPath);

                // Disk 삭제
                File newFile = new File(path);
                

//                newFile = newFile.replace(url, uploadPath);
                if (newFile.exists()) {
                    newFile.delete();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return result;
    }

    /**
     * @author 방선주
     * @since 2024.05.21
     * <p>
     * 파일 단일 조회
     */
    @Override
    public FileVO fileSelectOne(int fileId) throws Exception {
        return fileDAO.fileSelectOne(fileId);
    }

    @Override
    public String editorUploadImg(MultipartFile file) throws Exception {
        MultipartFile uploadFile = file;
        // 파일 명 생성
        String originalFileName = uploadFile.getOriginalFilename();
        String ext = originalFileName.substring(originalFileName.lastIndexOf("."));
        String maskNm = UUID.randomUUID() + ext; // 랜덤으로 파일명 생성
        // 파일 저장 경로
        String uploadPath = null;
//        String[] pathArray = locationPath.split(", ");
//        for (String path : pathArray) {
//            if (path.startsWith("file:///")) {
//                // "file:///" 부분을 제거하고 경로를 반환
//                uploadPath = path.replace("file:///", "") + editPath;
//                break; // 루프를 종료하여 첫 번째 일치하는 경로만 사용
//            }
//        }
//        if (uploadPath == null) {
//            throw new Exception("Upload path is not properly configured.");
//        }
        uploadPath = "/home/cloud-user/front-end/invest/uploadFiles" + editPath;
        String savePath = uploadPath + maskNm;
        // 파일 저장
        try {
            File saveFile = new File(savePath);
            if (!saveFile.exists()) {
                saveFile.mkdirs();
            }
            uploadFile.transferTo(saveFile);
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("File upload failed.");
        }
        String imgPath = frontUrl + "/invest/uploadFiles" + editPath + maskNm;
        return imgPath;
    }
}