package com.takensoft.common.filter;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.takensoft.cms.mber.dto.LoginDTO;
import com.takensoft.cms.mber.service.LgnHstryService;
import com.takensoft.cms.mber.service.RefreshTokenService;
import com.takensoft.cms.mber.vo.LgnHstryVO;
import com.takensoft.cms.mber.vo.MberVO;
import com.takensoft.cms.mber.vo.RefreshVO;
import com.takensoft.common.config.CommonConfig;
import com.takensoft.common.util.CommonUtils;
import com.takensoft.common.util.ErrorResponse;
import com.takensoft.common.util.JWTUtil;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author takensoft
 * @since 2024.04.01
 *
 * 로그인 필터
 */
public class LoginFilter extends UsernamePasswordAuthenticationFilter {

    private final AuthenticationManager authenticationManager;
    private final JWTUtil jwtUtil;
    private final RefreshTokenService refreshTokenService;
    private final LgnHstryService lgnHstryService;
    private final CommonUtils commonUtils;
    private final CommonConfig commonConfig;

    private static long JWT_ACCESSTIME; // access 토큰 유지 시간
    private static long JWT_REFRESHTIME; // refresh 토큰 유지 시간
    private static int COOKIE_TIME; // 쿠키 유지 시간

    public LoginFilter(AuthenticationManager authenticationManager, JWTUtil jwtUtil, RefreshTokenService refreshTokenService, LgnHstryService lgnHstryService, CommonUtils commonUtils,
                       CommonConfig commonConfig, @Value("${jwt.accessTime}")long aTime, @Value("${jwt.refreshTime}")long rTime, @Value("${cookie.time}")int ctime) {
        this.authenticationManager = authenticationManager;
        this.jwtUtil = jwtUtil;
        this.refreshTokenService = refreshTokenService;
        this.lgnHstryService = lgnHstryService;
        this.commonUtils = commonUtils;
        this.commonConfig = commonConfig;


        this.JWT_ACCESSTIME = aTime;
        this.JWT_REFRESHTIME = rTime;
        this.COOKIE_TIME = ctime;

        this.setRequiresAuthenticationRequestMatcher(new AntPathRequestMatcher("/mbr/loginProc.json","POST"));
    }

    /**
     * @author takensoft
     * @since 2024.04.03
     * 로그인 검증
     */
    @SneakyThrows
    @Override
    public Authentication attemptAuthentication(HttpServletRequest req, HttpServletResponse res) throws AuthenticationException {
        ObjectMapper mapper = new ObjectMapper();
        LoginDTO login = mapper.readValue(req.getInputStream(), LoginDTO.class);
        // 클라이언트에서 요청한 아이디와 비밀번호 추출
        String lgnId = login.getLgnId();
        String pswd = login.getPswd();

        // 스프링 시큐리티에서 username과 password를 검증하기 위해서는 token에 담아야 함
        UsernamePasswordAuthenticationToken authToken = new UsernamePasswordAuthenticationToken(lgnId, pswd, null);

        // token에 담은 검증을 위한 AuthenticationManager로 전달
        return authenticationManager.authenticate(authToken);
    }

    /**
     * @author takensoft
     * @since 2024.04.03
     * 로그인 성공시 실행하는 메서드 -> JWT 발급
     */
    @SneakyThrows
    @Override
    protected void successfulAuthentication(HttpServletRequest req, HttpServletResponse res, FilterChain chain, Authentication authentication) {
        MberVO mber = (MberVO) authentication.getPrincipal();

        // 로그인 이력 등록
        LgnHstryVO lgnHstryVO = new LgnHstryVO();
        lgnHstryVO.setLgnId(mber.getLgnId());
        if(mber.getAuthorities().stream().anyMatch(role -> role.getAuthority().equals("ROLE_ADMIN"))) {
            lgnHstryVO.setLgnType("0");
        } else {
            lgnHstryVO.setLgnType("1");
        }
        lgnHstryVO.setCntnIp(commonUtils.getIp(req));
        lgnHstryVO.setCntnOperSys(commonUtils.getOS(commonUtils.getUserAgent(req)));
        lgnHstryVO.setDeviceNm(commonUtils.getDevice(commonUtils.getUserAgent(req)));
        lgnHstryVO.setBrwsrNm(commonUtils.getBrowser(commonUtils.getUserAgent(req)));
        lgnHstryService.LgnHstrySave(lgnHstryVO);

        // 토큰 생성(access, refresh)
        String accessToken = jwtUtil.createJwt("Authorization", mber.getMbrId(), mber.getLgnId(), mber.getMbrNm(), (List) mber.getAuthorities(), JWT_ACCESSTIME);
        String refreshToken = jwtUtil.createJwt("refresh", mber.getMbrId(), mber.getLgnId(), mber.getMbrNm(), (List) mber.getAuthorities(), JWT_REFRESHTIME);

        // refreshToken이 현재 IP와 계정으로 등록되어 있는지 확인
        RefreshVO refresh = new RefreshVO();
        refresh.setMbrId(mber.getMbrId());

        // refresh 토큰이 현재 아이피와 아이디로 DB에 등록 되어 있다면
        if(refreshTokenService.findByCheckRefresh(req, refresh)) {
            refreshTokenService.delete(req, refresh);
        }
        // refreshToken DB 저장
        refresh.setToken(refreshToken);
        refreshTokenService.saveRefreshToken(req, res, refresh, JWT_REFRESHTIME);

        // 응답설정
        res.setHeader("Authorization", accessToken);
//        res.setHeader("refresh", refreshToken);
        // 쿠키 방식
        res.addCookie(jwtUtil.createCookie("refresh",refreshToken, COOKIE_TIME));
        res.setStatus(HttpStatus.OK.value());
    }

    /**
     * @author takensoft
     * @since 2024.04.04
     * 로그인 실패시 실행하는 메소드
     */
    @Override
    protected void unsuccessfulAuthentication(HttpServletRequest req, HttpServletResponse res, AuthenticationException failed) throws IOException, ServletException {

        ErrorResponse errorResponse = new ErrorResponse();
        // 에러 내용
        errorResponse.setMessage("아이디 혹은 비밀번호를 잘못 입력하셨습니다.");
        errorResponse.setPath(req.getRequestURI());
        errorResponse.setError(HttpStatus.UNAUTHORIZED.getReasonPhrase());
        errorResponse.setStatus(HttpStatus.UNAUTHORIZED.value());
        errorResponse.setTimestamp(LocalDateTime.now());

        // 문자 인코딩
        req.setCharacterEncoding("UTF-8");
        res.setCharacterEncoding("UTF-8");

        // 응답설정
        res.setContentType(MediaType.APPLICATION_JSON_VALUE);
        res.setStatus(HttpStatus.UNAUTHORIZED.value());
        res.getOutputStream().write(commonConfig.getObjectMapper().writeValueAsBytes(errorResponse));
    }
}
