package com.takensoft.portal.entInfo.web;

import com.takensoft.common.excel.service.ExcelService;
import com.takensoft.common.exception.IdDuplicationException;
import com.takensoft.common.util.ResponseData;
import com.takensoft.portal.entInfo.dto.BplcExcelDTO;
import com.takensoft.portal.entInfo.dto.EntInfoExcelDTO;
import com.takensoft.portal.entInfo.service.EntInfoService;
import com.takensoft.portal.entInfo.vo.EntInfoVO;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.nio.charset.Charset;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.03.20
 *
 * 기업정보 관련 컨트롤러
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/government/entInfo")
public class EntInfoController {
    private final EntInfoService entInfoService;
    private final ExcelService excelService;

    /**
     * @author 박정하
     * @since 2024.03.20
     * @param entInfoVO
     * @return
     * @throws Exception
     *
     * 기업정보 등록
     */
    @PostMapping(path = "/insertProc.file")
    public ResponseEntity<?> insertProc(@RequestPart EntInfoVO entInfoVO, @RequestPart(required = false) List<MultipartFile> insertFileList) throws Exception {
        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();

        // 사업자등록번호 중복 검사
        int isExistence = entInfoService.selectCntByCheckBrno(entInfoVO.getBrno());
        if (isExistence > 0) {
            // throw new IdDuplicationException("이미 존재하는 사업자등록번호입니다.");
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("이미 존재하는 사업자등록번호입니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }

        // 기업정보 등록
        HashMap<String, Object> result = entInfoService.entInfoInsert(entInfoVO, insertFileList);
        int insertResult = (int) result.get("result");

        if(insertResult > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 등록 처리되었습니다.");
            responseData.setData(result);
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else if(insertResult < 0) {
            responseData.setStatus(HttpStatus.PAYLOAD_TOO_LARGE);
            responseData.setMessage("업로드 가능한 파일 용량 제한을 초과했습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.PAYLOAD_TOO_LARGE);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("등록에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author 박정하
     * @since 2024.03.21
     * @param params
     * @return
     * @throws Exception
     *
     * 기업정보 목록 조회
     */
    @PostMapping(path = "/listProc.json")
    public ResponseEntity<?> listProc(@RequestBody Map<String, String> params) throws Exception {
        Map<String, Object> result = entInfoService.entInfoSelectList(params);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 기업정보 목록 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author 박정하
     * @since 2024.03.21
     * @param entInfoVO
     * @return
     * @throws Exception
     *
     * 기업정보 상세 조회
     */
    @PostMapping(path = "/detailProc.json")
    public ResponseEntity<?> detailProc(@RequestBody EntInfoVO entInfoVO) throws Exception {
        HashMap<String, Object> result = entInfoService.entInfoDetail(entInfoVO);
        
        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 기업정보 상세 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author 박정하
     * @since 2024.03.22
     * @param entInfoVO, insertFileList
     * @return
     * @throws Exception
     *
     * 기업정보 수정
     */
    @PostMapping(path = "/updateProc.file")
    public ResponseEntity<?> updateProc(@RequestPart EntInfoVO entInfoVO, @RequestPart(required = false) List<MultipartFile> insertFileList) throws Exception {
        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();

        // 사업자등록번호 중복 검사
        if (entInfoVO.getIsBrnoChange()) {
            int isExistence = entInfoService.selectCntByCheckBrno(entInfoVO.getBrno());
            if (isExistence > 0) {
//            throw new IdDuplicationException("이미 존재하는 사업자등록번호입니다.");
                responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
                responseData.setMessage("이미 존재하는 사업자등록번호입니다.");
                return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }

        int result = entInfoService.entInfoUpdate(entInfoVO, insertFileList);
        if(result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 기업정보 수정 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else if(result < 0) {
            responseData.setStatus(HttpStatus.PAYLOAD_TOO_LARGE);
            responseData.setMessage("업로드 가능한 파일 용량 제한을 초과했습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.PAYLOAD_TOO_LARGE);
        } else {
            responseData.setStatus(HttpStatus.BAD_REQUEST);
            responseData.setMessage("기업정보 등록 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * @author 박정하
     * @since 2024.03.22
     * @param entInfoVO
     * @return
     * @throws Exception
     *
     * 기업정보 삭제
     */
    @PostMapping(path = "/deleteProc.json")
    public ResponseEntity<?> deleteProc(@RequestBody EntInfoVO entInfoVO) throws Exception {
        int result = entInfoService.entInfoDelete(entInfoVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if(result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 기업정보가 삭제 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("기업정보 삭제에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author 박정하
     * @since 2024.03.25
     * @param response, entInfo, bplcList
     * @return
     * @throws Exception
     *
     * 기업정보 엑셀 다운로드
     */
    @RequestMapping(path = "/excelDownload.json")
    public void excelDownload(HttpServletResponse response, @RequestPart EntInfoExcelDTO entInfo, @RequestPart List<BplcExcelDTO> bplcList) throws Exception {
        excelService.entInfoExcelDownload(response, entInfo, bplcList);
    }

    /**
     * @author 박정하
     * @since 2024.06.26
     * @param response, params
     * @return
     * @throws Exception
     *
     * 기업정보 전체 엑셀 다운로드
     */
    @RequestMapping(path = "/excelDownloadAll.json")
    public void excelDownloadAll(HttpServletResponse response, @RequestBody HashMap<String, String> params) throws Exception {
        List<EntInfoExcelDTO> entInfoList = entInfoService.excelDownloadAll(params);
        excelService.entInfoExcelDownloadAll(response, entInfoList);
    }
}