package com.takensoft.portal.ivstDscsn.service.Impl;

import com.takensoft.common.Pagination;
import com.takensoft.common.file.service.FileMngService;
import com.takensoft.common.file.service.FileService;
import com.takensoft.common.file.vo.FileMngVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import com.takensoft.portal.entInfo.vo.EntInfoVO;
import com.takensoft.portal.ivstDscsn.dao.IvstDscsnDAO;
import com.takensoft.portal.ivstDscsn.service.IvstDscsnService;
import com.takensoft.portal.ivstDscsn.vo.IvstDscsnVO;
import com.takensoft.portal.rvwMttr.service.Impl.RvwMttrServiceImpl;
import com.takensoft.portal.rvwMttr.service.RvwMttrService;
import com.takensoft.portal.rvwMttr.vo.RvwMttrPrgrsVO;
import com.takensoft.portal.rvwMttr.vo.RvwMttrVO;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.03.26
 *
 * 투자상담 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * IvstDscsnService : 투자상담 인터페이스 상속
 */
@Service("ivstDscsnService")
@RequiredArgsConstructor
public class IvstDscsnServiceImpl extends EgovAbstractServiceImpl implements IvstDscsnService {
    private final JWTUtil jwtUtil;
    private final IdgenService ivstDscsnIdgn;
    private final IvstDscsnDAO ivstDscsnDAO;
    private final FileMngService fileMngService;
    private final FileService fileService;
    private final RvwMttrServiceImpl rvwMttrService;

    /**
     * @author 박정하
     * @since 2024.03.26
     *
     * 투자상담 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public HashMap<String, Object> ivstDscsnInsert(IvstDscsnVO ivstDscsnVO) throws Exception {
        // 아이디 생성
        String ivstDscsnId = ivstDscsnIdgn.getNextStringId();
        ivstDscsnVO.setIvstDscsnId(ivstDscsnId);

        // 작성자 생성
        String writer = jwtUtil.getWriter();
        ivstDscsnVO.setRgtr(writer);

        // 첨부파일 등록
        List<MultipartFile> insertFileList = ivstDscsnVO.getInsertFileList();
        if (insertFileList.size() > 0) {
            FileMngVO fileMngVO = new FileMngVO();

            // 파일매니저 등록
            HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, insertFileList);

            // 파일매니저 아이디 삽입
            ivstDscsnVO.setFileMngId(fileMng.get("fileMngId").toString());
        }

        // 검토사항 등록
        int ivstDscsnResult = ivstDscsnDAO.ivstDscsnInsert(ivstDscsnVO);

        // 결과 반환
        HashMap<String, Object> result = new HashMap<>();
        result.put("ivstDscsnId", ivstDscsnId);
        result.put("result", ivstDscsnResult);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.26
     *
     * 투자상담 목록 조회
     */
    @Override
    public Map<String, Object> ivstDscsnList(Map<String, String> params) throws Exception {
        Pagination search = new Pagination(0, params);
        int cnt = ivstDscsnDAO.ivstDscsnListCount(search);
        Pagination pagination = new Pagination(cnt, params);
        List<HashMap<String, Object>> list = ivstDscsnDAO.ivstDscsnList(pagination);

        // 결과 반환
        Map<String, Object> result = new HashMap<>();
        result.put("list", list);
        result.put("pagination", pagination);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.26
     *
     * 투자상담 상세 조회
     */
    @Override
    public IvstDscsnVO ivstDscsnDetail(IvstDscsnVO ivstDscsnVO) throws Exception {
        IvstDscsnVO newIvstDscsnVO = new IvstDscsnVO();

        // IvstDscsnId가 있는 경우
        if (ivstDscsnVO.getIvstDscsnId() != null) {
            newIvstDscsnVO = ivstDscsnDAO.selectIvstDscsnDetail(ivstDscsnVO.getIvstDscsnId());
            // FileMngId가 있는 경우
            if (newIvstDscsnVO.getFileMngId() != null) {
                List<HashMap<String, Object>> fileList = fileService.fileSelectList(newIvstDscsnVO.getFileMngId());
                newIvstDscsnVO.setFileList(fileList);
            }
        }
        // IvstDscsnId가 없는 경우
        else {
            newIvstDscsnVO.setUseYn("Y"); // 사용유무
        }

        return newIvstDscsnVO;
    }

    /**
     * @author 박정하
     * @since 2024.03.27
     *
     * 투자상담 수정
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int ivstDscsnUpdate(IvstDscsnVO ivstDscsnVO) throws Exception {
        // 수정자 생성
        String writer = jwtUtil.getWriter();
        ivstDscsnVO.setMdfr(writer);

        int result = 0;

        // 첨부파일 정보
        String fileMngId = ivstDscsnVO.getFileMngId();
        FileMngVO fileMngVO = new FileMngVO();
        List<MultipartFile> insertFileList = ivstDscsnVO.getInsertFileList();
        List<HashMap<String, Object>> deleteFileList = ivstDscsnVO.getDeleteFileList();

        // FileMngId가 있는 경우
        if (fileMngId != null) {
            fileMngVO.setFileMngId(ivstDscsnVO.getFileMngId());
            for (HashMap<String, Object> file : deleteFileList) {
                file.put("fileId", Integer.parseInt(file.get("fileId").toString()));
            }
            result += fileMngService.fileMngUpadate(fileMngVO, deleteFileList, insertFileList);
        }
        // FileMngId가 없는 경우
        else {
            if (insertFileList.size() > 0) {
                HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, insertFileList);
                result += (int)fileMng.get("fileMngResult");
                ivstDscsnVO.setFileMngId(fileMng.get("fileMngId").toString());
            }
        }

        // 투자상담 수정
        result += ivstDscsnDAO.ivstDscsnUpdate(ivstDscsnVO);

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.27
     *
     * 투자상담 삭제
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int ivstDscsnDelete(IvstDscsnVO ivstDscsnVO) throws Exception {
        // 수정자 생성
        String writer = jwtUtil.getWriter();
        ivstDscsnVO.setMdfr(writer);

        int result = 0;
        ivstDscsnVO.setUseYn("N");
        result += ivstDscsnDAO.ivstDscsnUpdate(ivstDscsnVO);

        // 검토사항 삭제
        result += rvwMttrService.rvwMttrDeleteByIvstDscsnId(ivstDscsnVO.getIvstDscsnId());

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.28
     *
     * 투자상담 최근 1건 조회
     */
    @Override
    public IvstDscsnVO ivstDscsnByNewOne(String entId) throws Exception {
        return ivstDscsnDAO.ivstDscsnByNewOne(entId);
    }

    /**
     * @author 박정하
     * @since 2024.06.10
     *
     * 투자상담 삭제 (기업정보 삭제용)
     */
    @Override
    public int ivstDscsnDeleteByEntId(String entId) throws Exception {
        int result = 0;

        List<IvstDscsnVO> ivstDscsnList = ivstDscsnDAO.ivstDscsnListByIvstDscsnId(entId);
        if (ivstDscsnList.size() > 0) {
            for (IvstDscsnVO ivstDscsnVO : ivstDscsnList) {
                result += ivstDscsnDelete(ivstDscsnVO);
            }
        }

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.06.12
     *
     * 투자상담 목록 조회 (기업정보 삭제용)
     */
    public List<IvstDscsnVO> ivstDscsnListByIvstDscsnId(String entId) throws Exception {
        return ivstDscsnDAO.ivstDscsnListByIvstDscsnId(entId);
    }
}